##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
__doc__='''DCO2 Database Connection

$Id: DA.py,v 1.7 2002/05/15 21:14:23 matt Exp $''' 
__version__='$Revision: 1.7 $'[11:-2]

from db import DB
import Shared.DC.ZRDB.Connection, sys, DABase, SP
from Globals import HTMLFile
from ImageFile import ImageFile
from ExtensionClass import Base
import string

manage_addZOracleConnectionForm=HTMLFile('dtml/connectionAdd',globals())
manage_addZOracleStoredProcedureForm=HTMLFile('dtml/ProcedureAdd',globals())

myname = string.split(__name__,'.')[1]
if myname == "ZOracleDA":
    dbstring = "Z Oracle"
    database_type = "Oracle"
else:
    dbstring = myname 
    database_type = myname

def manage_addZOracleConnection(self, id, title,
                                connection_string,
                                check=None, REQUEST=None):
    """Add a DB connection to a folder"""
    self._setObject(id, Connection(
	id, title, connection_string, check))
    if REQUEST is not None: return self.manage_main(self,REQUEST)

def manage_addZOracleStoredProcedure(self, id, title,
                                connection, procname, acquire,
                                check=None, REQUEST=None):
    """Add a Stored Procedure to a folder"""
    self._setObject(id, Procedure(id, title, connection, procname,
        acquire, check))
    if REQUEST is not None: return self.manage_main(self,REQUEST)


def DCO2Name(self, *ignored):
    "Return the dbstring we're running as"
    return dbstring

class Connection(DABase.Connection):
    " "
    database_type=database_type
    id='%s_database_connection' % dbstring
    meta_type=title='%s Database Connection' % dbstring
    icon='misc_/%s/conn' % myname

    def factory(self): return DB

    def table_info(self):
	return self._v_database_connection.table_info()

class Procedure(SP.Procedure):
    " "
    id='ZOracleStoredProcedure'
    meta_type=title="%s Stored Procedure" % dbstring
    icon="misc_/%s/proc" % myname

classes=('DA.Connection','DA.Procedure')

meta_types=(
    {'name':'%s Database Connection' % dbstring,
     'action':'manage_addZOracleConnectionForm',
     },
    {'name':'%s Stored Procedure' % dbstring,
     'action':'manage_addZOracleStoredProcedureForm',
     },
    )

folder_methods={
    'manage_addZOracleConnection':
    manage_addZOracleConnection,
    'manage_addZOracleConnectionForm':
    manage_addZOracleConnectionForm,
    'manage_addZOracleStoredProcedure':
    manage_addZOracleStoredProcedure,
    'manage_addZOracleStoredProcedureForm':
    manage_addZOracleStoredProcedureForm,
    'DCO2Name': DCO2Name,
    }

__ac_permissions__=(
    ('Add %s Database Connections' % dbstring,
     ('manage_addZOracleConnectionForm',
      'manage_addZOracleConnection')),
    ('Add %s Stored Procedures' % dbstring,
     ('manage_addZOracleStoredProcedureForm',
      'manage_addZOracleStoredProcedure')),
    ('Execute %s Stored Procedures' % dbstring, 
     ('manage_testZOracleStoredProcedure',"")),
    )

misc_={
    'conn':   ImageFile('Shared/DC/ZRDB/www/DBAdapterFolder_icon.gif'),
    }

for icon in ('table', 'view', 'stable', 'what',
	     'field', 'text','bin','int','float',
	     'date','time','datetime', 'proc'):
    misc_[icon]=ImageFile('icons/%s.gif' % icon, globals())
