import unittest

class connectTestCase(unittest.TestCase):
    def setUp(self):
        self.command = ""
        self.module  = ""
    
    def runTest(self):
        try:
            db = self.module.connect(self.command)
            assert hasattr(db,'close'), \
                "ERROR : connection object must have close"
            assert hasattr(db,'commit'), \
                "ERROR : connection object must have commit"
            assert hasattr(db,'rollback'), \
                "WARNING : no rollback (OPTIONAL)"
            try:
                c = db.cursor()
            except AttributeError:
                errormsg = "ERROR : " + str(self.module)
                errormsg = errormsg + " cursor function required"
                assert 0, errormsg
        except AttributeError:
            errormsg = "ERROR : " + str(self.module)
            errormsg = errormsg + " must implement a connect function"
            assert 0, errormsg
        
class apilevelTestCase(unittest.TestCase):
    def setUp(self):
        self.module     = ""
    
    def runTest(self):
        
        acceptable = ['1.0','2.0']
        
        try:
            apilevel = self.module.apilevel
            assert apilevel in acceptable, \
                 "ERROR : apilevel must be either '1.0' or '2.0'"
        except AttributeError:
            errormsg = "ERROR : " + str(self.module)
            errormsg = errormsg + "must provide an apilevel"
            assert 0, errormsg
            
class threadsafetyTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
    
    def runTest(self):
        acceptable = [0,1,2,3]
        try:
            thread = self.module.threadsafety
            assert thread in acceptable, \
                "ERROR : " + thread + " is not an acceptable thread level"
        except AttributeError:
            assert 0, "ERROR : self.module must have a threadsafety attribute"
    
class paramstyleTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        
    
    def runTest(self):
        acceptable = ['qmark',
                      'numeric',
                      'named',
                      'format',
                      'pyformat']
        try:
            styles = self.module.paramstyle()
            for style in styles:
                assert style in acceptable, \
                    "ERROR : " + style + " is not an acceptable paramstyle"
        except AttributeError:
            assert 0, "ERROR : self.module must implement a paramstyle function"

class ConnectionCloseTestCase(unittest.TestCase):
    def setUp(self):
        self.command    = ""
        self.module     = ""
        
    def runTest(self):
        try:
            connection = self.module.connect(self.command)
            try:
                connection.close()
            except AttributeError:
                assert 0, "ERROR : Connection object requires close function"
        except AttributeError:
            assert 0, "ERROR : Danger will robinson!"
    
class ConnectionCommitTestCase(unittest.TestCase):
    def setUp(self):
        self.command = ""
        self.module   = ""
    
    def runTest(self):
        try:
            connection = self.module.connect(self.command)
            try:
                connection.commit()
            except AttributeError:
                assert 0, "ERROR : Connection object requires commit function"
        except AttributeError:
            assert 0, "ERROR : module requires connect"
    
class ConnectionRollbackTestCase(unittest.TestCase):
    def setUp(self):
        self.command    = ""
        self.module     = ""
        
    def runTest(self):
        try:
            connection = self.module.connect(self.command)
            #cursor = connection.cursor()
            try:
                connection.rollback()
            except AttributeError, self.module.NotSupportedError:
                assert 0, "WARNING : Rollback not implemented"
            connection.close()
        except AttributeError:
            assert 0, "ERROR : module requires connect"
        
class ConnectionCursorTestCase(unittest.TestCase):
    def setUp(self):
        self.command = ""
        self.module  = ""
        
    def runTest(self):
        try:
            connection = self.module.connect(self.command)
            try:
                cursor = connection.cursor()
            except AttributeError:
                assert 0, "ERROR : Connection object requires cursor function"
            connection.close()
        except AttributeError:
            assert 0, "ERROR : module requires connect function"
        
class CursorDescriptionTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        try:
            connection  = self.module.connect(self.commands[0])
            cursor      = connection.cursor()
            
            try:
                descr = cursor.description
                if descr:
                    assert 0, "description should be none"
                cursor.execute(self.commands[1])
                descr = cursor.description
                if descr:
                    for x in descr:
                        if len(x) != 7:
                            assert 0, "description must be 7 items"
                            break
            except AttributeError:
                assert 0, \
                    "ERROR : cursor object requires description attribute"
            connection.close()
        except AttributeError:
            errormsg = "ERROR : something wrong with " + str(self.module)
            errormsg = errormsg + " or connection.cursor"
            assert 0, errormsg
    
class CursorRowcountTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        self.count      = 0
        
    def runTest(self):
        if not hasattr(self,"count"):
            self.count = 0
            
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        try:
            rowcount = cursor.rowcount
            assert rowcount==-1, \
                "ERROR : rowcount must be -1 for no actions"
            cursor.execute(self.commands[1])
            if not self.count:
                pass
                #if not cursor.rowcount() >= 0:
                #    assert 0, "rowcount failed"
            else:
                assert cursor.rowcount == self.count, \
                    "rowcount did not match self.count"
                
        except AttributeError:
            assert 0, \
                "ERROR : must provide rowcount function"
        
        connection.close()
        
class CursorCallprocTestCase(unittest.TestCase):
    def setUp(self):
        self.commands = []
        self.module   = ""
    def runTest(self):
        try:
            connection  = self.module.connect(self.commands[0])
            cursor      = connection.cursor()
            try:
                cursor.callproc(self.commands[1],self.commands[2])
                connection.close()
            except AttributeError:
                assert 0, str(self.module)+ " requires connect command"
        except AttributeError:
            assert 0, str(self.module)+ " requires connect command"
    
class CursorCloseTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection = self.module.connect(self.commands[0])
        cursor     = connection.cursor()
        try:
            cursor.close()
        except AttributeError:
            assert 0, \
                "ERROR : cursor object must provide close function"
                
        functions = ["fetchone","fetchmany","fetchall","nextset"]
        for function in functions:
            try:
                f = getattr(cursor,function)
                f()
                assert 0, \
                    "ERROR : close must render cursor functions inoperable"
            except self.module.Error:
                continue
        try:
            cursor.setinputsizes(1)
            assert 0, \
                    "ERROR : close must render cursor functions inoperable"
        except self.module.Error:
            pass
        try:
            cursor.setoutputsize(1)
            assert 0, \
                    "ERROR : close must render cursor functions inoperable"
        except self.module.Error:
            pass

        try:
            cursor.execute(self.commands[1])
            assert 0, \
                    "ERROR : close must render cursor functions inoperable"
        except self.module.Error:
            pass
        
        connection.close()
        
class CursorExecuteTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection = self.module.connect(self.commands[0])
        cursor = connection.cursor()
        try:
            for command in self.commands[1:]:
                cursor.execute(command)
        except AttributeError:
            assert 0, \
                "ERROR : cursor object requires execute function"
        connection.close()
    
class CursorExecutemanyTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        try:
            cursor.executemany(self.commands[1],self.commands[2])
        except AttributeError:
            assert 0, "ERROR : executemany not provided"
        connection.close()
    
class CursorFetchoneTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
    
    def runTest(self):
        connection = self.module.connect(self.commands[0])
        cursor = connection.cursor()
        try:
            cursor.execute(self.commands[1])
            try:
                result = cursor.fetchone()
            except AttributeError:
                assert 0, \
                    "ERROR : cursor object requires fetchone function"
        except AttributeError:
            assert 0, \
                "ERROR :  cursor object requires execute function"
        connection.close()
        
class CursorFetchmanyTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        cursor.arraysize = self.commands[1]
        try:
            cursor.execute(self.commands[2])
            try:
                result = cursor.fetchmany()
            except AttributeError:
                assert 0, \
                    "ERROR : cursor object requires fectchmany function"
        except AttributeError:
            assert 0, \
                "ERROR : cursor object requires execute function"
        connection.close()
    
class CursorFetchallTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        try:
            cursor.execute(self.commands[1])
            try:
                result = cursor.fetchall()
            except AttributeError:
                assert 0, \
                    "ERROR : cursor object requires fetchall function"
        except AttributeError:
            assert 0, \
                "ERROR : cursor object requires execute function"
        connection.close()
    
class CursorNextsetTestCase(unittest.TestCase):
    def setUp(self):
        self.commands = []
        self.module   = ""
        
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.close()
        try:
            cursor.execute(self.commands[1])
            result1 = cursor.fetchone()
            cursor.nextset()
            result2 = cursor.fetchone()
            if result2:
                assert result1 != result2, \
                    "ERROR : " + str(self.module) + "failed in nextset"
        except AttributeError:
            assert 0, \
                "WARNING : " + str(self.module) + " failed on nextset"
        connection.close()
    
class CursorArraysizeTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        try:
            cursor.arraysize = self.commands[1]
        except AttributeError:
            assert 0, \
                "ERROR : " + str(self.module) + "failed on arraysize"
        connection.close()
    
class CursorSetinputsizeTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        try:
            cursor.setinputsizes(self.commands[1])
        except AttributeError:
            assert 0, "WARNING : setinputsizes failed"
        connection.close()
    
class CursorSetoutputsizeTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
        
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        try:
            cursor.setoutputsize(self.commands[1])
        except AttributeError:
            assert 0, "WARNING : setoutputsize failed"
        connection.close()
    
class DateTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        self.year   = 0
        self.month  = 0
        self.day    = 0
    
    def runTest(self):
        try:
            self.module.Date(self.year,self.month,self.day)
        except AttributeError, self.module.NotSupportedError:
            assert 0, "ERROR " + str(self.module) + " does not support Date"
    
class TimeTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        self.hour   = 0
        self.minute = 0
        self.second = 0
    
    def runTest(self):
        try:
            self.module.Time(self.hour,self.minute,self.second)
        except AttributeError, self.module.NotSupportedError:
            assert 0, "ERROR " + str(self.module) + "does not support Time"
    
class TimestampTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        self.year   = 0
        self.month  = 0
        self.day    = 0
        self.hour   = 0
        self.minute = 0
        self.second = 0
        
    def runTest(self):
        try:
            self.module.Timestamp(self.year,
                                  self.month,
                                  self.day,
                                  self.hour,
                                  self.minute,
                                  self.second)
        except AttributeError, self.module.NotSupportedError:
            assert 0, "ERROR " + str(self.module) + "does not support Timestamp"
    
class TimeFromTicksTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        self.ticks  = 0
    
    def runTest(self):
        try:
            self.module.TimeFromTicks(self.ticks)
        except AttributeError, self.module.NotSupportedError:
            assert 0, "ERROR " + str(self.module) + "does not support TimeFromTicks"
    
class DateFromTicksTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        self.ticks  = 0
    
    def runTest(self):
        try:
            self.module.DateFromTicks(self.ticks)
        except AttributeError, self.module.NotSupportedError:
            assert 0, "ERROR " + str(self.module) + "does not support DateFromTicks"
    
class TimestampFromTicksTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        self.ticks  = 0
    
    def runTest(self):
        try:
            self.module.TimestampFromTicks(self.ticks)
        except AttributeError, self.module.NotSupportedError:
            assert 0, "ERROR " + str(self.module) + "does not support TimestampFromTicks"
    
class BinaryTestCase(unittest.TestCase):
    def setUp(self):
        self.module = ""
        self.string = ""
    
    def runTest(self):
        try:
            self.module.Binary(self.string)
        except AttributeError, self.module.NotSupportedError:
            assert 0, "ERROR " + str(self.module) + "does not support Binary"

class STRINGTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
    
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        cursor.execute(self.commands[1])
        descr = cursor.description[self.commands[2]][1]
        assert descr == self.module.STRING, \
            "ERROR : " + str(descr) + " is not " + str(self.module) + ".STRING"
        connection.close()
        
class BINARYTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
    
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        cursor.execute(self.commands[1])
        descr = cursor.description[self.commands[2]][1]
        assert descr == self.module.BINARY, \
            "ERROR : " + str(descr) + " is not " + str(self.module) + ".BINARY"
        connection.close()
        
class NUMBERTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
    
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        cursor.execute(self.commands[1])
        descr = cursor.description[self.commands[2]][1]
        assert descr == self.module.NUMBER, \
            "ERROR : " + str(descr) + " is not " + str(self.module) + ".NUMBER"
        connection.close()
        
class DATETIMETestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
    
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        cursor.execute(self.commands[1])
        descr = cursor.description[self.commands[2]][1]
        assert descr == self.module.DATETIME, \
            "ERROR : " + str(descr) + " is not " + str(self.module) + ".DATETIME"
        connection.close()
        
class ROWIDTestCase(unittest.TestCase):
    def setUp(self):
        self.commands   = []
        self.module     = ""
    
    def runTest(self):
        connection  = self.module.connect(self.commands[0])
        cursor      = connection.cursor()
        cursor.execute(self.commands[1])
        descr = cursor.description[self.commands[2]][1]
        assert descr == self.module.ROWID, \
            "ERROR : " + str(descr) + " is not " + str(self.module) + ".ROWID"
        connection.close()
